import time
import functools
import sys
from importlib import reload


def get_current_fn(ext):
    import os

    filename = os.path.basename(__file__)
    return filename[: -(len(ext) + 1)]


def show_gpu():
    import torch

    if torch.cuda.is_available():
        print(f"There is/are {torch.cuda.device_count()} gpus.")
        for i in range(torch.cuda.device_count()):
            print(f"the model of card {i} is {torch.cuda.get_device_name(i)}")
        print("for more info, run nvidia-smi")
    if torch.backends.mps.is_available():
        print("Mac gpu is available")
    else:
        print("no gpu available")


def hello_world():
    print("hello this is nlpbaselines!")


def timer(func):
    @functools.wraps(func)
    def time_closure(*args, **kwargs):
        start = time.perf_counter()
        result = func(*args, **kwargs)
        time_elapsed = time.perf_counter() - start
        print(f"Function: {func.__name__}, Time: {time_elapsed}")
        return result

    return time_closure


def reimport_all(package):
    reload(sys.modules[package])


def dump_pickle(obj, fn):
    """_summary_

    Args:
        obj (the object to dump): _description_
        fn (filename): _description_
    """
    import pickle

    with open(fn, "wb") as file:
        # A new file will be created
        pickle.dump(obj, file)


def read_pickle(fn):
    # read a pickle file
    import pickle

    with open(fn, "rb") as file:
        # The protocol version used is detected automatically, so we do not
        # have to specify it.
        obj = pickle.load(file)
    return obj


# get the first argument of the command line


def get_first_arg():
    import sys

    return sys.argv[1]


if __name__ == "__main__":
    print(get_current_fn("py"))



import random
import numpy as np
from transformers import (
    AutoModelForSequenceClassification,
    AutoTokenizer,
    Trainer,
    TrainingArguments,
)
from datasets import Dataset
from sklearn.metrics import accuracy_score, recall_score, f1_score
import pandas as pd
import torch
import gc
import time
from torch.nn import DataParallel
import torch
import os
from sklearn.model_selection import KFold
from datetime import datetime



class DatasetLoader:
    def __init__(self, df_type="pandas", text_col="text", label_col="label"):
        self.text_col = text_col
        self.label_col = label_col
        self.df_type = df_type

    def load_dataset(self, dataframe):
        # Split the DataFrame into training and validation sets
        dataframe.rename(columns={self.text_col: "text"}, inplace=True)
        train_df = dataframe[dataframe["split"] == "train"]
        val_df = dataframe[dataframe["split"] == "validation"]
        # get the number of unique labels
        num_labels = len(train_df[self.label_col].unique())
        train_dataset = Dataset.from_pandas(train_df)
        val_dataset = Dataset.from_pandas(val_df)
        print(
            f"\nDataset loaded. The dataset has {num_labels} labels, {len(train_df)} training items, {len(val_df)} validation items. \n{dataframe.head(3)}"
        )
        return train_dataset, val_dataset

    def load_dataset_cv(self, dataframe, n_splits=5):
        dataframe.rename(columns={self.text_col: "text"}, inplace=True)
        kf = KFold(n_splits=n_splits)
        num_labels = len(dataframe[self.label_col].unique())
        folds = []
        for train_index, val_index in kf.split(dataframe):
            train_df = dataframe.iloc[train_index]
            val_df = dataframe.iloc[val_index]
            train_dataset = Dataset.from_pandas(train_df)
            val_dataset = Dataset.from_pandas(val_df)
            folds.append((train_dataset, val_dataset))
        print(
            f"\nDataset loaded. The dataset has {num_labels} labels, {n_splits} splits were made. Each split has {len(train_df)} training items and {len(val_df)} test items.\n{dataframe.head(3)}"
        )
        return folds


class Classifier:
    def __init__(
        self,
        model_name="camembert/camembert-base-ccnet-4gb",
        num_labels=2,
        use_multi_gpu=False,
    ):
        self.model_name = model_name
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        self.device = "cuda" if torch.cuda.is_available() else "cpu"
        self.device = "mps" if torch.backends.mps.is_available() else "cpu"
        self.f1 = None
        self.accuracy = None
        self.recall = None
        self.use_multi_gpu = use_multi_gpu
        self.num_labels = num_labels
        self.model = AutoModelForSequenceClassification.from_pretrained(
            self.model_name, num_labels=self.num_labels
        )
        self.model_size = self.get_model_size(self.model)[1]
        self.model = None
        print(
            f"\nModel loaded. We will finetune {self.model_name} with {self.num_labels} labels."
        )
        self.all_f1_scores = []
        self.all_accuracy_scores = []
        self.all_recall_scores = []

        # Additional configurations can be added here

    def get_model_size(self, model):
        # from https://camembert-model.fr/posts/tutorial/
        param_size = 0
        param_count = 0
        for param in model.parameters():
            param_count += param.nelement()
            param_size += param.nelement() * param.element_size()
        buffer_size = 0
        for buffer in model.buffers():
            buffer_size += buffer.nelement() * buffer.element_size()
        size_all_mb = (param_size + buffer_size) / 1024**2
        return param_count, f"{size_all_mb:.2f}MB"

    def tokenize_function(self, examples):
        if "camembert" in self.model_name:
            return self.tokenizer(
                examples["text"], padding="max_length", truncation=True, max_length=512
            )
        else:
            return self.tokenizer(
                examples["text"], padding="max_length", truncation=True
            )

    def compute_metrics(self, pred):
        labels = pred.label_ids
        preds = pred.predictions.argmax(-1)
        accuracy = accuracy_score(labels, preds)
        recall = recall_score(labels, preds, average="weighted")
        f1 = f1_score(labels, preds, average="weighted")
        accuracy = float("{:.2f}".format(accuracy))
        recall = float("{:.2f}".format(recall))
        f1 = float("{:.2f}".format(f1))
        self.f1 = f1
        self.accuracy = accuracy
        self.recall = recall
        self.all_f1_scores.append(f1)
        self.all_accuracy_scores.append(accuracy)
        self.all_recall_scores.append(recall)
        return {"accuracy": accuracy, "recall": recall, "f1": f1}

    def train(
        self,
        train_dataset,
        validation_dataset,
        epochs=2,
        batch_size=10,
        learning_rate=2e-5,
        output_dir="./results/",
    ):
        model = AutoModelForSequenceClassification.from_pretrained(
            self.model_name, num_labels=self.num_labels
        )
        if self.use_multi_gpu and torch.cuda.device_count() > 1:
            print(f"Using {torch.cuda.device_count()} GPUs")
            model = DataParallel(self.model)
        model.to(self.device)
        dstrain_encoded = train_dataset.map(
            self.tokenize_function, batched=True, batch_size=None
        )
        dsvalidation_encoded = validation_dataset.map(
            self.tokenize_function, batched=True, batch_size=None
        )
        print("data encoded")
        training_args = TrainingArguments(
            output_dir=output_dir,
            per_device_train_batch_size=batch_size,
            per_device_eval_batch_size=batch_size,
            num_train_epochs=epochs,
            seed=42,
            learning_rate=learning_rate,
            weight_decay=0.01,
            logging_strategy="epoch",
            evaluation_strategy="epoch",
            save_strategy = "no"
        )

        # training_args.set_save(strategy="epoch")

        trainer = Trainer(
            model=model,
            args=training_args,
            train_dataset=dstrain_encoded,
            eval_dataset=dsvalidation_encoded,
            compute_metrics=self.compute_metrics,
            tokenizer=self.tokenizer,
        )
        trainer.train()
        trainer.save_model(output_dir)
        del trainer, model
        # free gpu memory
        gc.collect()
        torch.cuda.empty_cache()

    def train_cv(self, folds, epochs=2, batch_size=10, learning_rate=2e-5, output_dir="./results/"):
        for i, (train_dataset, validation_dataset) in enumerate(folds):
            print(f"Training on fold {i+1}/{len(folds)}")
            self.train(
                train_dataset, validation_dataset, epochs, batch_size, learning_rate, output_dir+f"fold-{i+1}/"
            )



# for each folder an output file

# folders
import os
import multiprocessing as mp
from collections import defaultdict
from glob import glob
import pickle
import re

folders = glob("./*/")
print(folders)
corrupt_files = defaultdict(list)


def check_folder(fd):
    files = glob(fd+"*.pickle")
    log_fn = fd.split("/")[-2] + ".txt"
    if os.path.exists(log_fn):
        os.remove(log_fn)
    for fn in files:
        # read pickled and check it's corrupted
        with open(fn, "rb") as handler:
            try:
                # print("ok")
                p_file = pickle.load(handler)
                with open(log_fn, 'a') as f:
                    if len(p_file) != 10000 and len(p_file) != 8304:
                        f.write(str(len(p_file))+"\n")
                        # don't retain
                        f.write(re.sub(r"[^\d]", "", fn.split("/")[-1])+"\n")
            except:
                print(fn.split("/")[-1])


with mp.Pool(16) as p:
    p.map(check_folder, folders)



import os
# import language_tool_python


# class ErrorParser:
#     def __init__(self, language):
#         self.parser = language_tool_python.LanguageTool(language)

#     def parse_errors(self, text):
#         return self.parser.check(text)

#     def show_errors(self, text):
#         obj = self.parse_errors(text)
#         self.analyze_obj(obj)

#     def analyze_obj(self, obj):
#         if obj:
#             for error in obj:
#                 context = error.context[error.offsetInContext:(
#                     error.offsetInContext+error.errorLength)]
#                 if len(error.replacements) > 0:
#                     replacement = error.replacements[0]
#                 else:
#                     replacement = "no suggestion"
#                 rule_id = error.ruleId
#                 sentence = error.sentence
#                 category = error.category
#                 print(f'category: {category}, {rule_id}')
#                 # print(f"erroneous part: {context}")
#                 print(f'error: {error}')
#                 # print(f'sentence: {sentence}\n')
#                 # print(f"suggestion: {replacement}")
#                 # print(f"message: {message}")
#         else:
#             print("No error detected.")


# def log_errors(path, obj, fn, total_words):
#     # output = defaultdict(list)
#     os.mkdir(path+"error_stat/")
#     os.mkdir(path+"error_example/")
#     output_file = path+"error_stat/"+fn
#     error_word_ratio = len(obj)/total_words
#     with open(output_file, 'a+') as f:
#         with open(path+"error_example/"+fn, "a+") as g:
#             for error in obj:
#                 context = error.context[error.offsetInContext:(
#                     error.offsetInContext+error.errorLength)]
#                 if len(error.replacements) > 0:
#                     replacement = error.replacements[0]
#                 else:
#                     replacement = "no suggestion"
#                 sentence = error.sentence
#                 category = error.category
#                 message = error.message
#                 # write the error example
#                 print(f'{category}', file=g)
#                 print(f'{error}', file=g)
#                 print(f'{sentence}\n', file=g)
#                 print(f"{fn}", file=f, end="")
#                 print(f"\t{len(obj)}", file=f, end="")
#                 print(f"\t{total_words}", file=f, end="")
#                 print(f"\t{error_word_ratio}", file=f, end="")
#                 print(f"\t{context}", file=f, end="")
#                 print(f"\t{replacement}", file=f, end="")
#                 print(f"\t{sentence}", file=f, end="")
#                 print(f"\t{category}", file=f, end="")
#                 print(f"\t{message}", file=f)


# if __name__ == "__main__":
#     text_tense = "Hier soir je vois une voiture rose dans la rue."
#     # errors = parse_errors(parser, text_tense)
#     # print(errors)
#     # show_errors(errors )



import glob
# from frenchnlp import files

txts = glob.glob("*.txt")

total = []
for t in txts:
    total += list(open(t, "r"))
    total = [x.strip() for x in total if x.strip() != "2"]

print(list(set(total)))


# print(len(set(total)))



# table for the pilot study on french models
import pandas as pd
import torch
import random
from classifier import Classifier, DatasetLoader
import gc
import time

def get_current_fn(ext):
    import os
    filename = os.path.basename(__file__)
    return filename[: -(len(ext) + 1)]

def log_model():
    log = pd.DataFrame(
        columns=[
            "model_name",
            "f1",
            "accuracy",
            "recall",
            "training_time",
            "data_size",
            "batch_size",
            "learning_rate",
            "epochs",
        ]
    )
    return log


# create a function to add data to the log
def add_to_log(
    log,
    model_name,
    f1,
    accuracy,
    recall,
    training_time,
    data_size,
    batch_size,
    learning_rate,
    epochs,
):
    log.loc[len(log) + 1] = [
        model_name,
        f1,
        accuracy,
        recall,
        training_time,
        data_size,
        batch_size,
        learning_rate,
        epochs,
    ]
    return log


# Toy data
data = {
    "text": [f"This is sentence {i}" for i in range(30)],
    "label": [random.randint(0, 1) for _ in range(30)],
    "split": ["train" if i < 25 else "validation" for i in range(30)],
}
df = pd.DataFrame(data)

# Initialize the classifier
loader = DatasetLoader(text_col="text", label_col="label")
folds = loader.load_dataset_cv(df, n_splits=5)

print("data ok")

# create a pandas dataframe to log model name, f1, accuracy, recall, training time, data_size, batch size, learning rate, epochs

log = log_model()


# check the number of available gpus
print(f"Number of GPUs: {torch.cuda.device_count()}")
for i in range(torch.cuda.device_count()):
    print(f"GPU {i}: {torch.cuda.get_device_name(i)}")


# model_lists = [
#     "bert-base-uncased",
#     "distilbert-base-uncased",
#     "bert-base-multilingual-uncased",
#     "camembert-base",
#     "camembert/camembert-large",
#     "camembert/camembert-base-ccnet",
#     "camembert/camembert-base-ccnet-4gb",
#     "camembert/camembert-base-oscar-4gb",
#     "camembert/camembert-base-wikipedia-4gb",
#     "flaubert/flaubert_small_cased",
#     "flaubert/flaubert_base_uncased",
#     "flaubert/flaubert_base_cased",
#     "flaubert/flaubert_large_cased",
# ]

model_lists = ["distilbert-base-uncased"]
filename = get_current_fn("py")
from datetime import datetime

current_datetime = datetime.now()
formatted_datetime = current_datetime.strftime("%Y-%m-%d-%H-%M-%S")

for m in model_lists:
    print(f"model {m} start")
    epoch = 2
    batch_size = 10
    learning_rate = 2e-5
    classifier = Classifier(model_name=m, num_labels=2, use_multi_gpu=False)
    start_time = time.time()
    classifier.train_cv(
        folds,
        epochs=epoch,
        batch_size=batch_size,
        learning_rate=learning_rate,
        output_dir=f"./results/{filename}-{formatted_datetime}/{m}/",
    )
    print("F1 Scores for each fold:", classifier.all_f1_scores)
    print("Accuracy Scores for each fold:", classifier.all_accuracy_scores)
    print("Recall Scores for each fold:", classifier.all_recall_scores)

    end_time = time.time()
    training_time = f"{end_time - start_time:.2f}"
    log = add_to_log(
        log,
        m,
        classifier.all_f1_scores,
        classifier.all_accuracy_scores,
        classifier.all_recall_scores,
        training_time,
        len(df),
        10,
        2e-5,
        epoch,
    )
    del classifier
    gc.collect()
    torch.cuda.empty_cache()
    # for slurm
    print(f"model {m} done, results saved to {filename}.csv")
    log.to_csv(f"{filename}.csv", index=False)



from files import read_file
import re


def count_words(file, punctuation=False):
    if punctuation:
        return len(read_file(file).split())
    else:
        return len(re.findall(r'\S+', read_file(file)))

if __name__ == "__main__":

    print(count_words("test.txt" print(read_file("test.txt").split())
    # print(word_counter)
    # test = list(open("test.txt", 'r'))

    # print(read_file("test.txt"))
# else:
    # print("Executed when imported")



from svd2vec import svd2vec
import pickle
import os
import glob

import multiprocessing as mp
import time

start = time.time()
# print(f"{mp.cpu_count()} cpus")

number_of_cores = mp.cpu_count()

os.system(f"echo {number_of_cores} cpus allocated")

fns = glob.glob(
    "/Users/xiaoou/Documents/projects/frwac_articles_pickles/*.pickle")

# for fn in fns:
total_documents = []
for i, fn in enumerate(fns):
    # os.system(f'echo {fns} loaded')
    if i < 1:
        with open(fn, 'rb') as f:
            documents = pickle.load(f)
        break

texts = [a[1].split() for a in documents]
# os.system(f"echo total documents = {len(total_documents)}")
print(len(texts))
print(texts[0])
raise Exception("strop")
# creating the words representation (can take a while)
os.system("echo training started")
svd = svd2vec(texts, window=2, verbose=True, workers=number_of_cores)
os.system("echo training completed")
svd.save("svd_1.bin")
os.system("echo model 1 saved")
svd.save_word2vec_format("svd_1.txt")
os.system("echo model 2 saved")
end = time.time()
os.system(f"echo {(end-start)/60}")



import multiprocessing as mp


def f_vs_files(f, fns):
    workers = mp.cpu_count()
    with mp.Pool(workers) as p:
        p.map(f, fns)


def nlp_stanza(text):
    nlp = stanza.Pipeline("fr", logging_level='WARN', use_gpu=False)
    doc = nlp(text)
    for sent in doc.sentences:
        with open(text+".txt", 'w') as f:
            f.write(sent.text)


def nlp_stanza2(nlp, text):
    doc = nlp(text)
    for sent in doc.sentences:
        with open(text+".txt", 'w') as f:
            f.write(sent.text)


if __name__ == "__main__":
    import stanza
    from itertools import repeat
    nlp = stanza.Pipeline("fr", logging_level='WARN', use_gpu=False)
    texts = ["la dd", "je suis beau"]
    # f_vs_files(nlp_stanza2, zip(repeat(nlp),texts))
    workers = mp.cpu_count()
    with mp.Pool(workers) as p:
        p.starmap(nlp_stanza2, zip(repeat(nlp), texts))



from setuptools import setup, find_packages

with open("README.md") as readme_file:
    README = readme_file.read()

with open("HISTORY.md") as history_file:
    HISTORY = history_file.read()

setup_args = dict(
    name="nlpbaselines",
    version="0.0.28",
    description="Establish quick baselines for NLP tasks",
    long_description_content_type="text/markdown",
    long_description=README,
    include_package_data=True,
    license="Apache Licence 2.0",
    packages=find_packages(),
    author="Xiaoou WANG",
    author_email="xiaoouwangfrance@gmail.com",
    keywords=["text mining", "npl", "corpus", "french", "bert", "transformers"],
    url="https://github.com/xiaoouwang/nlpbaselines",
    download_url="https://pypi.org/project/nlpbaselines/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

install_requires = [
    "accelerate",
    "datasets",
    "matplotlib",
    "torch",
    "transformers",
    "scikit-learn",
    "numpy",
    "pandas",
    "py7zr",
    "sentencepiece",
    "sacremoses",
    "numba"
]


if __name__ == "__main__":
    setup(**setup_args, install_requires=install_requires)



import bz2
import os
import _pickle as cPickle
import py7zr
import pickle


def remove_exist(fn):
    if os.path.exists(fn):
        os.remove(fn)


def extract_7z(fn):
    with py7zr.SevenZipFile(fn, mode='r') as z:
        z.extractall("./fr_wac/")


def load_compressed_pickle(file):
    with bz2.BZ2File(file, 'rb') as f:
        return cPickle.load(file)


def dump_compressed_pickle(fn, data):
    with bz2.BZ2File(fn + '.pbz2', 'w') as f:
        cPickle.dump(data, f)


def dump_pickle(fn, obj):
    with open(fn + '.pickle', 'wb') as handle:
        pickle.dump(obj, handle, protocol=pickle.HIGHEST_PROTOCOL)


def load_pickle(fn):
    with open(fn, 'rb') as handle:
        return pickle.load(handle)


def write_file(path, content):
    with open(path, 'w') as f:
        f.write(content)


def read_file(path):
    try:
        with open(path, "r") as f:
            return f.read()
    except:
        print("file doesn't exist")


def iter_documents(top_directory, ext):
    """Iterate over all documents and apply the function f"""
    for r, d, files in os.walk(top_directory):
        # print(files)
        for file in filter(
                lambda file: file.endswith(ext), files):
            yield os.path.join(r, file)
        # f(file)
